grammar for values zinc =

precedences
  precedence LET;
  precedence WHERE;
  precedence DEFINE;
  right ";";
  precedence LIST;
  precedence IF;
  right ":=" "<-";
  left "as";
  left "|";
  right ",";
  right "or";
  right "&";
  left  "not";
  left  "<" ">" "=" ">=" "<=" "<>" "==" "!=";
  right "@" "^";
  right "::";
  left  "+" "-";
  precedence TYPEARROW;
  left  "*" "/" INFIX;
  precedence UMINUS;
  precedence APP;
  left ".";
  right "!";

(************ Une phrase d'une implementation ********************)

rule entry Implementation =
  parse Expr e; ";;"
          -> make_impl(Zexpr e)
      | "let"; Binding_list L; ";;" with precedence LET
          -> make_impl(Zletdef(false, L))
      | "let"; "rec"; Binding_list L; ";;" with precedence LET
          -> make_impl(Zletdef(true, L))
      | "type"; Type_decl tydecl; ";;"
          -> make_impl(Ztypedef tydecl)
      | "exception"; Exc_decl excdecl; ";;"
          -> make_impl(Zexcdef excdecl)
      | "#"; Directive d; ";;"
          -> make_impl(Zimpldirective d)
      | EOF _
          -> raise (io_failure "eof")

(************ Une phrase d'une interface **************************)

and entry Interface =
  parse "value"; Value_decl valdecl; ";;"
          -> make_intf(Zvaluedecl valdecl)
      | "type"; Type_decl tydecl; ";;"
          -> make_intf(Ztypedecl tydecl)
      | "exception"; Exc_decl excdecl; ";;"
          -> make_intf(Zexcdecl excdecl)
      | "#"; Directive d; ";;"
          -> make_intf(Zintfdirective d)
      | EOF _
          -> raise (io_failure "eof")

(**** Doit etre place devant expr pour des raisons de priorite ****)

and Simple_expr_list =
  parse Simple_expr e; Simple_expr_list el
          -> e :: el
      | Simple_expr e
          -> [e]

and Expr_comma_list =
  parse Expr e; ","; Expr_comma_list el
          -> e :: el
      | Expr e	with precedence ","
          -> [e]

and Expr_sm_list =
  parse Expr e with precedence LIST -> [e]
      | Expr_sm_list el; ";"; Expr e with precedence LIST -> e :: el

and Expr_label =
  parse Ext_ident gr; "="; Expr e -> (find_label gr, e) 

and Expr_label_list =
  parse Expr_label e with precedence LIST -> [e]
      | Expr_label_list el; ";"; Expr_label e with precedence LIST -> e :: el

(*********  EXPRESSIONS  ***************************************************)

and entry Expr =
  parse Simple_expr e
          -> e
      | Simple_expr e; Simple_expr_list el with precedence APP
          -> make_apply (e, el)
      | Expr e; ","; Expr_comma_list el
          -> make_tuple_expr(e::el)
      | "-"; Expr e1 with precedence UMINUS
          -> make_unary_minus e1
      | "not"; Expr e
          -> make_unop "not" e
      | Ide id; "<-"; Expr e
          -> make_expr (Zassign(id, e))
      | Expr e1; INFIX i; Expr e2
          -> make_binop i e1 e2
      | Expr e1; "*"; Expr e2
          -> make_binop "*" e1 e2
      | Expr e1; "/"; Expr e2
          -> make_binop "/" e1 e2
      | Expr e1; "+"; Expr e2
          -> make_binop "+" e1 e2
      | Expr e1; "-"; Expr e2
          -> make_binop "-" e1 e2
      | Expr e1; "::"; Expr e2
          -> make_expr(Zconstruct(constr_cons, [e1;e2]))
      | Expr e1; "@"; Expr e2
          -> make_binop "@" e1 e2
      | Expr e1; "^"; Expr e2
          -> make_binop "^" e1 e2
      | Expr e1; "<"; Expr e2
          -> make_binop "<" e1 e2
      | Expr e1; ">"; Expr e2
          -> make_binop ">" e1 e2
      | Expr e1; "="; Expr e2
          -> make_binop "=" e1 e2
      | Expr e1; "=="; Expr e2
          -> make_binop "==" e1 e2
      | Expr e1; ">="; Expr e2
          -> make_binop ">=" e1 e2
      | Expr e1; "<="; Expr e2
          -> make_binop "<=" e1 e2
      | Expr e1; "<>"; Expr e2
          -> make_binop "<>" e1 e2
      | Expr e1; "!="; Expr e2
          -> make_binop "!=" e1 e2
      | Expr e1; ":="; Expr e2
          -> make_binop ":=" e1 e2
      | Expr e1; "&"; Expr e2 
          -> make_expr(Zsequand(e1,e2))
      | Expr e1; "or"; Expr e2
          -> make_expr(Zsequor(e1,e2))
      | Simple_expr e1; "."; Ext_ident gr; "<-"; Expr e2
          -> make_expr(Zrecord_update(e1, find_label gr, e2))
      | Simple_expr e1; ".("; Expr e2; ")"; "<-"; Expr e3
          -> make_ternop "vect_assign" e1 e2 e3
      | "if"; Expr e1; "then"; Expr e2; "else"; Expr e3 with precedence IF
          -> make_expr(Zcondition(e1,e2,e3))
      | "if"; Expr e1; "then"; Expr e2 with precedence IF
          -> make_expr(Zcondition(e1,e2,
                                  make_expr(Zconstruct(constr_void,[]))))
      | "while"; Expr e1; "do"; Expr e2; "done"
          -> make_expr(Zwhile(e1,e2))
      | "for"; Ide id; "="; Expr e1; "to"; Expr e2; "do"; Expr e3; "done"
          -> make_expr(Zfor(id, e1, e2, true, e3))
      | "for"; Ide id; "="; Expr e1; "down"; "to";
        Expr e2; "do"; Expr e3; "done"
          -> make_expr(Zfor(id, e1, e2, false, e3))
      | Expr e1; ";"; Expr e2
          -> make_expr(Zsequence(e1,e2))

      | "match"; Expr e; "with"; Function_match M
          -> make_expr(Zapply(make_expr(Zfunction M), [e]))
      | "let"; Binding_list L; "in"; Expr e with precedence LET
          -> make_expr(Zlet(false,L,e))
      | "let"; "rec"; Binding_list L; "in"; Expr e with precedence LET
          -> make_expr(Zlet(true,L,e))
      | "fun"; Fun_match M
          -> make_expr(Zfunction M)
      | "function"; Function_match M
          -> make_expr(Zfunction M)
      | "try"; Expr e; "with"; Try_match M
	  -> make_expr(Ztrywith(e,M))

      | Expr e; "where"; Binding_list L	 with precedence WHERE
          -> make_expr(Zlet(false,L,e))
      | Expr e; "where"; "rec"; Binding_list L	with precedence WHERE
          -> make_expr(Zlet(true,L,e))

and Simple_expr =
  parse Struct_constant sc
          -> make_expr(Zconstant sc)
      | Ext_ident gr
          -> expr_constr_or_ident gr
      | "("; ")"
          -> make_expr(Zconstruct(constr_void,[]))
      | "["; Expr_sm_list el; "]"
          -> make_list el
      | "["; "]"
          -> make_expr(Zconstruct (constr_nil, []))
      | "[|"; Expr_sm_list el; "|]"
          -> make_expr(Zvector(rev el))
      | "[|"; "|]"
          -> make_expr(Zvector [])
      | "("; Expr e; ":"; Type ty; ")"
          -> make_expr(Zconstraint(e,ty))
      | "("; Expr e; ")"
          -> e
      | "begin"; Expr e; "end"
          -> e
      | "{"; Expr_label_list el; "}"
          -> make_expr (Zrecord el)
      | "!"; Simple_expr e1
          -> make_unop "!" e1
      | Simple_expr e; "."; Ext_ident gr
          -> make_expr(Zrecord_access(e, find_label gr))
      | Simple_expr e1; ".("; Expr e2; ")" with precedence "."
          -> make_binop "vect_item" e1 e2

and entry Struct_constant =
  parse Atomic_constant ac
          -> SCatom ac

and entry Atomic_constant =
  parse INT i
          -> ACint i
      | FLOAT f
          -> ACfloat f
      | STRING s
          -> ACstring s
      | CHAR c
          -> ACchar c

(**************     Bindings divers	 ************************************)

and Fun_match =
  parse Simple_pattern_list pl; "->"; Expr e; "|"; Fun_match M
          -> (pl,e) :: M
      | Simple_pattern_list pl; "->"; Expr e
	  -> [pl,e]

and Function_match =
  parse Pattern p; "->"; Expr e; "|"; Function_match M
          -> ([p],e) :: M
      | Pattern p; "->"; Expr e
          -> [[p],e]

and Try_match =
  parse Pattern p; "->"; Expr e; "|"; Try_match M
          -> (p,e) :: M
      | Pattern p; "->"; Expr e
          -> [p,e]

and Binding_list =
  parse Binding b; "and"; Binding_list bl
          -> b :: bl
      | Binding b
          -> [b]

and Binding =
  parse Pattern p; "="; Expr e	with precedence DEFINE
          -> (p, e, ref 0)
      | Ide s; Simple_pattern_list pl; "="; Expr e  with precedence DEFINE
          -> (make_pat(Zvarpat s), make_expr(Zfunction [pl,e]), ref 0)

(**************************   Patterns	*************************************)

and Pattern_sm_list =
  parse Pattern p; ";"; Pattern_sm_list pl
          -> make_pat(Zconstructpat(constr_cons, [p;pl]))
      | Pattern p
          -> make_pat(Zconstructpat(constr_cons,
                                   [p; make_pat(Zconstructpat(constr_nil,[]))]))

and Pattern_label_list =
  parse Pattern_label pl; ";"; Pattern_label_list pll
          -> pl::pll
      | Pattern_label pl
          -> [pl]
      | "_"
          -> []

and Pattern_label =
  parse Ext_ident gr; "="; Pattern p -> (find_label gr, p) 

and Pattern_constr_list =
  parse "("; Pattern_comma_list pl; ")"
          -> pl
      | Simple_pattern p
          -> [p]

and Pattern_comma_list =
  parse Pattern p; ","; Pattern_comma_list pl
          -> p :: pl
      | Pattern p  with precedence ","
          -> [p]
  
and Simple_pattern_list =
  parse Simple_pattern p; Simple_pattern_list pl
          -> p :: pl
      | Simple_pattern p
          -> [p]

and entry Pattern =
  parse Simple_pattern p
          -> p
      | Pattern p; "as"; IDENT s
          -> make_pat(Zaliaspat(p,s))
      | Pattern p1; "::"; Pattern p2
          -> make_pat(Zconstructpat(constr_cons, [p1;p2]))
      | Pattern p1; ","; Pattern_comma_list pl
          -> make_tuple_pat(p1::pl)
      | Ext_ident c; Pattern_constr_list pl
          -> make_pat(Zconstructpat (find_constructor c, pl))
      | Pattern p1; "|"; Pattern p2
          -> make_pat(Zorpat(p1,p2))

and Simple_pattern =
  parse Atomic_constant ac
          -> make_pat(Zconstantpat ac)
      | "-"; INT i
          -> make_pat(Zconstantpat(ACint(minus_int i)))
      | "-"; FLOAT f
          -> make_pat(Zconstantpat(ACfloat(minus_float f)))
      | "_"
          -> make_pat(Zwildpat)
      | Ide s
          -> pat_constr_or_var s
      | Qual_ident q
          -> make_pat(Zconstructpat(find_constructor (GRmodname q),[]))
      | "("; ")"
          -> make_pat(Zconstructpat(constr_void,[]))
      | "["; "]"
          -> make_pat(Zconstructpat(constr_nil,[]))
      | "["; Pattern_sm_list p; "]"
          -> p
      | "("; Pattern p; ":"; Type ty; ")"
          -> make_pat(Zconstraintpat(p,ty))
      | "{"; Pattern_label_list pl; "}"
          -> make_pat(Zrecordpat pl)
      | "("; Pattern p; ")"
          -> p

(***************************** Noms de globaux ***************************)

and entry Ide =
  parse IDENT s                                         -> s
      | "prefix"; Infx s                                -> s

and Infx =
    parse INFIX i -> i
        | "+" -> "+"            | "-" -> "-"
        | "*" -> "*"            | "/" -> "/"
        | "<=" -> "<="          | ">=" -> ">="
        | "<>" -> "<>"          | "<" -> "<"
        | ">" -> ">"            | "::" -> "::"
        | "@" -> "@"            | "&" -> "&"
        | "or" -> "or"          | "^" -> "^"
        | "!=" -> "!="          | ":=" -> ":="
        | "==" -> "=="          | "=" -> "="
        | "not" -> "not"        | "!" -> "!"

and entry Qual_ident =
  parse IDENT modname; "__"; Ide s -> {qual=modname; id=s}

and entry Ext_ident =
  parse Qual_ident q -> GRmodname q
      | Ide s -> GRname s

(**************************** Les expressions de type *********************)

and entry Type =
  parse Simple_type ty
          -> ty
      | Type ty1; "*"; Type_star_list tyl
          -> make_typ(Ztypeconstr(gr_type_tuple, ty1::tyl))
      | Type ty1; "->"; Type ty2 with precedence TYPEARROW
          -> make_typ(Ztypearrow(ty1, ty2))

and entry Simple_type =
  parse Type_var tv
          -> make_typ(Ztypevar tv)
      | Ext_ident gr
          -> make_typ(Ztypeconstr(gr, []))
      | Simple_type ty; Ext_ident gr
          -> make_typ(Ztypeconstr(gr, [ty]))
      | "("; Type_comma_list tyl; ")"; Ext_ident gr
          -> make_typ(Ztypeconstr(gr, tyl))
      | "("; Type ty; ")"
          -> ty

and Type_star_list =
  parse Simple_type ty -> [ty]
      | Simple_type ty1; "*"; Type_star_list tyl -> ty1::tyl

and Type_var =
  parse "'"; IDENT s -> s

and Type_comma_list =
  parse Type ty1; ","; Type ty2; ( * (parse ","; Type ty -> ty)) tyl
     -> ty1::ty2::tyl

and entry Type_expr =
  parse Type ty
           -> (reset_type_expression_vars [];
               push_type_level ();
               let t = type_of_type_expression false ty in
                 pop_type_level ();
                 generalize_type t;
                 t)

(******************** Les declarations **************)

and entry Value_decl =
  parse Value1_decl vd1;
        ( * (parse "and"; Value1_decl vd -> vd)) vdl -> vd1::vdl

and entry Type_decl =
  parse Type1_decl td1;
        ( * (parse "and"; Type1_decl td -> td)) tdl     -> td1::tdl

and entry Exc_decl =
  parse Constr_decl exc1;
        ( * (parse "and"; Constr_decl exc -> exc)) excl -> exc1::excl

and Value1_decl =
  parse Ide s; ":"; Type ty -> (s,ty, ValueNotPrim)
      | Ide s; ":"; Type ty; "="; Prim_decl p -> (s,ty,p)

and Prim_decl =
  parse INT arity; STRING name
          -> find_primitive arity name ""
      | INT arity; STRING name; STRING flags
          -> find_primitive arity name flags

and Type1_decl =
  parse Type_params params; IDENT ty_name; Type1_def ty_def
          -> (ty_name, params, ty_def)

and Type1_def =
  parse (* epsilon *)
          -> Zabstract_type
      | "="; Constr_decl cd1; ( * (parse "|"; Constr_decl cd -> cd)) cdl
          -> Zvariant_type (cd1::cdl)
      | "="; "{"; Label_decl ld1; 
        ( * (parse ";"; Label_decl ld -> ld)) ldl; "}"
          -> Zrecord_type (ld1::ldl)
      | "=="; Type ty
          -> Zabbrev_type ty

and Constr_decl =
  parse IDENT s; "of"; Constr_args tyl
          -> (s,tyl)
      | IDENT s
          -> (s,[])

and Label_decl =
  parse Mutable_option m; IDENT s; ":"; Type ty -> (s, ty, m)

and Constr_args =
  parse Constr_arg arg -> [arg]
      | Constr_arg arg; "*"; Constr_args arg_list -> arg :: arg_list

and Constr_arg =
  parse Mutable_option m; Simple_type ty -> (ty, m)

and Mutable_option =
  parse "mutable" -> Mutable
      | (* epsilon *)-> Notmutable

and Type_params =
  parse "("; Type_var tv1;
        ( * (parse ","; Type_var tv -> tv)) tvl; ")"    -> tv1::tvl
      | Type_var tv                                     -> [tv]
      |                                                 -> []

(************************ Les directives ******************************)

and Directive =
  parse IDENT id; STRING arg -> Zdir(id,arg)
;;
