echo_values false;;

mlet (Syntax(_,_,Scan_syntax(
 	((strings: (string * num) list),
	 (doubles: (num * (num * string) list) list),
	 (Num_tok, Int_tok, Float_tok, Ident_tok,
          Bool_tok, String_tok, Infix_tok, Char_tok, Eof_tok),
	 (strdelim,cmtdelim),
	 _)),_)) = syntax_of("zinc",false);;
mlet zinc_INT = string_of_num Int_tok
 and zinc_FLOAT = string_of_num Float_tok
 and zinc_IDENT = string_of_num Ident_tok
 and zinc_STRING = string_of_num String_tok
 and zinc_CHAR = string_of_num Char_tok
 and zinc_INFIX = string_of_num Infix_tok
 and zinc_EOF = string_of_num Eof_tok
;;
mlet other_rules =
  implode(
   map_succeed
    (fun (name,tok) ->
      if is_legal_ident name then failwith "on continue" else
        "  | \"" ^ string_for_read name ^ "\" -> " ^ string_of_num tok ^ "\n")
    strings)
;;
        

let ld = <:lexdef:Lexer<
  rule Main = parse
    [` ` `\010` `\013` `\009` `\026` `\012`] + ->
      Main ()
  | ([`A`-`Z` `a`-`z`] | `_` [`A`-`Z` `a`-`z` `'` `0`-`9`])
    ( `_` ? [`A`-`Z` `a`-`z` `'` `0`-`9`] ) * ->
      let s = get_lexeme () in
       (try
          find_keyword s
        with failure "find" ->
          lex_val := Repr s;
          if is_infix s then ##zinc_INFIX else ##zinc_IDENT)
  | (([`0`-`9`]+ (`.` [`0`-`9`]+)? [`e` `E`] (`-`)? [`0`-`9`]+)
          |([`0`-`9`]+ `.` [`0`-`9`]+ ([`e` `E`] (`-`)? [`0`-`9`]+)?)) ->
      lex_val := Repr(float_of_string(get_lexeme ()));
      ##zinc_FLOAT
  | [`0`-`9`]+ ->
      lex_val := Repr(int_of_string(get_lexeme ()));
      ##zinc_INT
  | `"` ->
      reset_string_buffer();
      String ()
  | "`" [^ `\\`] "`" ->
      lex_val := Repr(get_lexeme_char 1);
      ##zinc_CHAR
  | "`\\" [`\\` `\`` `n` `t` `b` `r`] "`" ->
      lex_val := Repr(char_for_backslash (get_lexeme_char 2)); ##zinc_CHAR
  | "`\\" [`0`-`9`] [`0`-`9`] [`0`-`9`] "`" ->
      lex_val := Repr(char_for_decimal_code 2); ##zinc_CHAR
  | "(*" ->
      comment_depth := 1; Comment ()
##other_rules
  | `\000` -> ##zinc_EOF
  | _ ->
      prerr_begline "## Illegal character : ";
      prerr_endline (char_for_read(get_lexeme_char 0));
      raise Toplevel

and Comment = parse
    "(*" ->
      comment_depth := succ !comment_depth; Comment ()
  | "*)" ->
      comment_depth := pred !comment_depth;
      if !comment_depth == 0 then Main () else Comment ()
  | `"` ->
      reset_string_buffer();
      String ();
      reset_string_buffer();
      Comment ()
  | `\000` -> (* end of file *)
      prerr_begline "## Comment not terminated.";
      prerr_endline "";
      raise Toplevel
  | _ ->
      Comment ()

and String = parse
    `"` ->
      lex_val := Repr(get_stored_string());
      ##zinc_STRING
  | "\\\n" ->
      String()
  | `\\` [`\\` `"` `n` `t` `b` `r`] ->
      store_string_char(char_for_backslash(get_lexeme_char 1)); String ()
  | `\\` [`0`-`9`] [`0`-`9`] [`0`-`9`] ->
      store_string_char(char_for_decimal_code 1); String ()
  | `\000` -> (* end of file *)
      prerr_begline "## String not terminated.";
      prerr_endline "";
      raise Toplevel
  | _ ->
      store_string_char(get_lexeme_char 0); String ()
>>;;

let oc = open_out "lex_auto.ml";;

open_user_print oc;;

print_string "#open \"obj\";;
#open \"misc\";;
#open \"lex_run\";;
#open \"lex_aux\";;";
print_newline();
print_MLsyntax(MLdecl(translate_lexdef ld));
print_string ";;";
print_newline();;

let (Syntax(_,_,Scan_syntax(
 	((strings: (string * num) list),
	 (doubles: (num * (num * string) list) list),
	 (Num_tok, Int_tok, Float_tok, Ident_tok,
          Bool_tok, String_tok, Infix_tok, Char_tok, Eof_tok),
	 (strdelim,cmtdelim),
	 _)),_)) = syntax_of("zinc",false);;
print_string "let keyword_list = [";
print_newline();
let rec print_keywords flag = function
    [] -> ()
  | (name, (Int tok))::rest ->
      if is_legal_ident name then
        (if flag then (print_string ";"; print_newline());
         print_string "\""; print_string_for_read name;
         print_string "\", "; print_int tok;
         print_keywords true rest)
      else
         print_keywords flag rest
in print_keywords false strings;
print_string "];;";
print_newline();;
print_string "let token_end_of_file = ";
print_int ##zinc_EOF;
print_string "\nand token_end_of_phrase = ";
print_int (int_of_num (assoc ";;" strings));
print_string ";;";
print_newline();;

close_user_print oc;;
close_out oc;;

echo_values true;;
